@@ -23,12 +23,13 @@ from __future__ import absolute_import
 import sys
 import types
 import warnings
+from copy import deepcopy
 from logging import getLogger
 
 import numpy as np
+
 from pykdtree.kdtree import KDTree
-from pyresample import _spatial_mp, data_reduce, geometry
-from pyresample import CHUNK_SIZE
+from pyresample import CHUNK_SIZE, _spatial_mp, data_reduce, geometry
 
 logger = getLogger(__name__)
 
@@ -36,6 +37,10 @@ try:
     from xarray import DataArray
     import dask.array as da
     import dask
+    if hasattr(dask, 'blockwise'):
+        blockwise = da.blockwise
+    else:
+        blockwise = da.atop
 except ImportError:
     DataArray = None
     da = None
@@ -887,13 +892,13 @@ def get_sample_from_neighbour_info(resample_type, output_shape, data,
 
 
 def lonlat2xyz(lons, lats):
-
     R = 6370997.0
-    x_coords = R * da.cos(da.deg2rad(lats)) * da.cos(da.deg2rad(lons))
-    y_coords = R * da.cos(da.deg2rad(lats)) * da.sin(da.deg2rad(lons))
-    z_coords = R * da.sin(da.deg2rad(lats))
+    x_coords = R * np.cos(np.deg2rad(lats)) * np.cos(np.deg2rad(lons))
+    y_coords = R * np.cos(np.deg2rad(lats)) * np.sin(np.deg2rad(lons))
+    z_coords = R * np.sin(np.deg2rad(lats))
 
-    return da.stack(
+    stack = np.stack if isinstance(lons, np.ndarray) else da.stack
+    return stack(
         (x_coords.ravel(), y_coords.ravel(), z_coords.ravel()), axis=-1)
 
 
@@ -904,7 +909,7 @@ def query_no_distance(target_lons, target_lats, valid_output_index,
     """Query the kdtree. No distances are returned.
 
     NOTE: Dask array arguments must always come before other keyword arguments
-          for `da.atop` arguments to work.
+          for `da.blockwise` arguments to work.
 
     """
     voi = valid_output_index
@@ -917,7 +922,7 @@ def query_no_distance(target_lons, target_lats, valid_output_index,
 
     coords = lonlat2xyz(target_lons_valid, target_lats_valid)
     distance_array, index_array = kdtree.query(
-        coords.compute(),
+        coords,
         k=neighbours,
         eps=epsilon,
         distance_upper_bound=radius,
@@ -941,6 +946,20 @@ def query_no_distance(target_lons, target_lats, valid_output_index,
     return res_ia
 
 
+def _my_index(index_arr, vii, data_arr, vii_slices=None, ia_slices=None,
+              fill_value=np.nan):
+    """Helper function for 'get_sample_from_neighbour_info'."""
+    vii_slices = tuple(
+        x if x is not None else vii.ravel() for x in vii_slices)
+    mask_slices = tuple(
+        x if x is not None else (index_arr == -1) for x in ia_slices)
+    ia_slices = tuple(
+        x if x is not None else index_arr for x in ia_slices)
+    res = data_arr[vii_slices][ia_slices]
+    res[mask_slices] = fill_value
+    return res
+
+
 class XArrayResamplerNN(object):
     def __init__(self,
                  source_geo_def,
@@ -1011,11 +1030,11 @@ class XArrayResamplerNN(object):
             args = (mask, dims, self.valid_input_index, dims)
         # res.shape = rows, cols, neighbors
         # j=rows, i=cols, k=neighbors, m=source rows, n=source cols
-        res = da.atop(query_no_distance, 'jik', tlons, 'ji', tlats, 'ji',
-                      valid_oi, 'ji', *args, kdtree=resample_kdtree,
-                      neighbours=self.neighbours, epsilon=self.epsilon,
-                      radius=self.radius_of_influence, dtype=np.int,
-                      new_axes={'k': self.neighbours}, concatenate=True)
+        res = blockwise(query_no_distance, 'jik', tlons, 'ji', tlats, 'ji',
+                        valid_oi, 'ji', *args, kdtree=resample_kdtree,
+                        neighbours=self.neighbours, epsilon=self.epsilon,
+                        radius=self.radius_of_influence, dtype=np.int,
+                        new_axes={'k': self.neighbours}, concatenate=True)
         return res, None
 
     def get_neighbour_info(self, mask=None):
@@ -1141,12 +1160,12 @@ class XArrayResamplerNN(object):
         ia_slices = []
         # whether we have seen the geo dims in our analysis
         geo_handled = False
-        # dimension indexes for da.atop
+        # dimension indexes for da.blockwise
         src_adims = []
         flat_adim = []
-        # map source dimension name to dimension number for da.atop
+        # map source dimension name to dimension number for da.blockwise
         src_dim_to_ind = {}
-        # destination array dimension indexes for da.atop
+        # destination array dimension indexes for da.blockwise
         dst_dims = []
         for i, dim in enumerate(data.dims):
             src_dim_to_ind[dim] = i
@@ -1164,25 +1183,13 @@ class XArrayResamplerNN(object):
                 ia_slices.append(slice(None))
                 src_adims.append(i)
                 dst_dims.append(dim)
-        # map destination dimension names to atop dimension indexes
+        # map destination dimension names to blockwise dimension indexes
         dst_dim_to_ind = src_dim_to_ind.copy()
         dst_dim_to_ind['y'] = i + 1
         dst_dim_to_ind['x'] = i + 2
         # FUTURE: when we allow more than one neighbor
         # neighbors_dim = i + 3
 
-        def _my_index(index_arr, vii, data_arr, vii_slices=None,
-                      ia_slices=None, fill_value=np.nan):
-            vii_slices = tuple(
-                x if x is not None else vii.ravel() for x in vii_slices)
-            mask_slices = tuple(
-                x if x is not None else (index_arr == -1) for x in ia_slices)
-            ia_slices = tuple(
-                x if x is not None else index_arr for x in ia_slices)
-            res = data_arr[vii_slices][ia_slices]
-            res[mask_slices] = fill_value
-            return res
-
         new_data = data.data.reshape(flat_src_shape)
         vii = vii.ravel()
         dst_adims = [dst_dim_to_ind[dim] for dim in dst_dims]
@@ -1191,19 +1198,19 @@ class XArrayResamplerNN(object):
         # dst_adims.append(neighbors_dim)
         # ia_adims.append(neighbors_dim)
         # FUTURE: when we allow more than one neighbor we need to add
-        #         the new axis to atop:
+        #         the new axis to blockwise:
         #         `new_axes={neighbor_dim: self.neighbors}`
         # FUTURE: if/when dask can handle index arrays that are dask arrays
-        #         then we can avoid all of this complicated atop stuff
-        res = da.atop(_my_index, dst_adims,
-                      ia, ia_adims,
-                      vii, flat_adim,
-                      new_data, src_adims,
-                      vii_slices=vii_slices, ia_slices=ia_slices,
-                      fill_value=fill_value,
-                      dtype=new_data.dtype, concatenate=True)
+        #         then we can avoid all of this complicated blockwise stuff
+        res = blockwise(_my_index, dst_adims,
+                        ia, ia_adims,
+                        vii, flat_adim,
+                        new_data, src_adims,
+                        vii_slices=vii_slices, ia_slices=ia_slices,
+                        fill_value=fill_value,
+                        dtype=new_data.dtype, concatenate=True)
         res = DataArray(res, dims=dst_dims, coords=coords,
-                        attrs=data.attrs.copy())
+                        attrs=deepcopy(data.attrs))
 
         return res
 
